// SuDoku.h: interface for the CSuDoku class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SUDOKU_H__65E6101B_9489_4B80_9CDE_989DC8AD9EE5__INCLUDED_)
#define AFX_SUDOKU_H__65E6101B_9489_4B80_9CDE_989DC8AD9EE5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>

#define DIGIT_UNKNOWN -1
#define DIGIT_IMPOSSIBLE -2

enum eRestrictions {
	RESTRICTING_BY_X = 0,
	RESTRICTING_BY_Y,
	RESTRICTING_BY_SQUARE
};

class CSuDoku   // class to solve 3x3 Su Doku problems
{
public:
	CSuDoku();
	virtual ~CSuDoku();

	void ClearEverything();
	void InitializeXY(int iStartingValues[9][9]); // starting values can be 1-9 or DIGIT_UNKNOWN
	void InitializeYX(int iStartingValues[9][9]); // starting values can be 1-9 or DIGIT_UNKNOWN
	bool Solve(std::vector<CString>& vstrOutput);
	bool IsComplete();
	bool IsConsistent();
	int GetValueXY(int x, int y); // x and y in the range 1-9; returns 1-9 or DIGIT_UNKNOWN
	int GetValueYX(int y, int x); // x and y in the range 1-9; returns 1-9 or DIGIT_UNKNOWN
	unsigned char GetValueXYAsChar(int x, int y); // x and y in the range 1-9; returns '1'-'9' or ' '
	unsigned char GetValueYXAsChar(int y, int x); // x and y in the range 1-9; returns '1'-'9' or ' '
	int GetStartingValueXY(int x, int y); // x and y in the range 1-9; returns 1-9 or DIGIT_UNKNOWN
	int GetStartingValueYX(int y, int x); // x and y in the range 1-9; returns 1-9 or DIGIT_UNKNOWN
	unsigned char GetStartingValueXYAsChar(int x, int y); // x and y in the range 1-9; returns '1'-'9' or ' '
	unsigned char GetStartingValueYXAsChar(int y, int x); // x and y in the range 1-9; returns '1'-'9' or ' '

private:
	void UpdateKnownValues();
	int SquareNumber(int x, int y);
	void SquareExtremes(int square, int& minx, int& maxx, int& miny, int& maxy);
	bool ScanMatrixSimply(std::vector<CString>& vstrOutput); // returns true iff improvement
	bool ScanMatrixAdvanced(std::vector<CString>& vstrOutput, int iGroupSizeForAdvancedStuff = -1); // returns true iff improvement; iGroupSizeForAdvancedStuff = -1 or 2-8 (see code)
	void ImproveAsMuchAsPossibleBySimpleScanning(std::vector<CString>& vstrOutput);
	void Assume(int x, int y, int digit, std::vector<CString>& vstrOutput);
	void Restrict(int x, int y, int digit, std::vector<CString>& vstrOutput, enum eRestrictions eRestrictingHow, bool& bImproved);
	void RestrictAnythingBut(int x, int y, int digit, std::vector<CString>& vstrOutput, bool& bImproved);
	void RestrictToSet(int x, int y, std::vector<int> vPossibleDigits, std::vector<CString>& vstrOutput, enum eRestrictions eRestrictingHow, bool& bImproved);

	int m_iStartingValues[9][9]; // x, y
	int m_iKnownValues[9][9]; // x, y
	bool m_bPossible[9][9][10]; // x, y, digit
	bool m_bAllowedToGuess[9][9]; // x, y
	int m_iRecursionLevel;
	bool m_bIsComplete;
	bool m_bIsConsistent;
};

#endif // !defined(AFX_SUDOKU_H__65E6101B_9489_4B80_9CDE_989DC8AD9EE5__INCLUDED_)
